@@ -147,7 +147,7 @@ USER_OBJECT_
 }
 
 /* reason: GtkSignalFunc is a generic user-func, cannot auto-convert
-    (even though at least in the defs it is always treated as a GtkCallback)
+   (even though at least in the defs it is always treated as a GtkCallback)
 USER_OBJECT_
  S_gtk_toolbar_append_element ( USER_OBJECT_ s_object, USER_OBJECT_ s_type, USER_OBJECT_ s_widget, USER_OBJECT_ s_text, USER_OBJECT_ s_tooltip_text, USER_OBJECT_ s_tooltip_private_text, USER_OBJECT_ s_icon, USER_OBJECT_ s_callback, USER_OBJECT_ s_user_data )
 {
@@ -368,13 +368,16 @@ S_gtk_message_dialog_new(USER_OBJECT_ s_parent, USER_OBJECT_ s_flags, USER_OBJEC
         GtkWidget* ans;
         USER_OBJECT_ _result = NULL_USER_OBJECT;
 
-        ans = gtk_message_dialog_new(parent, flags, type, buttons, message_format);
+        if (!message_format)
+          message_format = "";
+        ans = gtk_message_dialog_new(parent, flags, type, buttons, "%s",
+                                     message_format);
 
         _result = toRPointerWithSink(ans, "GtkWidget");
 
         return(_result);
 }
-/* reason: same asC above */
+/* reason: same as above */
 USER_OBJECT_
 S_gtk_message_dialog_new_with_markup(USER_OBJECT_ s_parent, USER_OBJECT_ s_flags, USER_OBJECT_ s_type, USER_OBJECT_ s_buttons, USER_OBJECT_ s_message_format)
 {
@@ -387,13 +390,16 @@ S_gtk_message_dialog_new_with_markup(USER_OBJECT_ s_parent, USER_OBJECT_ s_flags
         GtkWidget* ans;
         USER_OBJECT_ _result = NULL_USER_OBJECT;
 
-        ans = gtk_message_dialog_new_with_markup(parent, flags, type, buttons, message_format);
+        if (!message_format)
+          message_format = "";
+        ans = gtk_message_dialog_new_with_markup(parent, flags, type, buttons,
+                                                 "%s", message_format);
 
         _result = toRPointerWithSink(ans, "GtkWidget");
 
         return(_result);
 }
-/* reason: same asC above */
+/* reason: same as above */
 USER_OBJECT_
 S_gtk_message_dialog_format_secondary_text(USER_OBJECT_ s_object, USER_OBJECT_ s_message_format)
 {
@@ -402,12 +408,14 @@ S_gtk_message_dialog_format_secondary_text(USER_OBJECT_ s_object, USER_OBJECT_ s
 
         USER_OBJECT_ _result = NULL_USER_OBJECT;
 
-        gtk_message_dialog_format_secondary_text(object, message_format);
+        if (!message_format)
+          message_format = "";
+        gtk_message_dialog_format_secondary_text(object, "%s", message_format);
 
 
         return(_result);
 }
-/* reason: same asC above */
+/* reason: same as above */
 USER_OBJECT_
 S_gtk_message_dialog_format_secondary_markup(USER_OBJECT_ s_object, USER_OBJECT_ s_message_format)
 {
@@ -416,7 +424,10 @@ S_gtk_message_dialog_format_secondary_markup(USER_OBJECT_ s_object, USER_OBJECT_
 
         USER_OBJECT_ _result = NULL_USER_OBJECT;
 
-        gtk_message_dialog_format_secondary_markup(object, message_format);
+        if (!message_format)
+          message_format = "";
+        gtk_message_dialog_format_secondary_markup(object, "%s",
+                                                   message_format);
 
 
         return(_result);
@@ -424,15 +435,17 @@ S_gtk_message_dialog_format_secondary_markup(USER_OBJECT_ s_object, USER_OBJECT_
 
 /* reason: var args, take lists of labels and responses from R */
 USER_OBJECT_
-S_gtk_dialog_add_buttons(USER_OBJECT_ s_object, USER_OBJECT_ s_labels, USER_OBJECT_ s_responses)
+S_gtk_dialog_add_buttons(USER_OBJECT_ s_object, USER_OBJECT_ s_labels,
+                         USER_OBJECT_ s_responses)
 {
-        USER_OBJECT_ _result = NULL_USER_OBJECT;
-		gint i;
-		
-		for (i = 0; i < GET_LENGTH(s_labels); i++)
-			S_gtk_dialog_add_button(s_object, VECTOR_ELT(s_labels, i), VECTOR_ELT(s_responses, i));
-
-        return(_result);
+  USER_OBJECT_ _result = NULL_USER_OBJECT;
+  gint i;
+  
+  for (i = 0; i < GET_LENGTH(s_labels); i++)
+    S_gtk_dialog_add_button(s_object, STRING_ELT(s_labels, i),
+                            VECTOR_ELT(s_responses, i));
+  
+  return(_result);
 }
 
 /* reason: var args, take lists of labels and responses describing the buttons */
@@ -471,7 +484,8 @@ S_gtk_recent_chooser_dialog_new_for_manager(USER_OBJECT_ s_title, USER_OBJECT_ s
   GtkWidget* ans;
   USER_OBJECT_ _result = NULL_USER_OBJECT;
 
-  ans = gtk_recent_chooser_dialog_new_for_manager(title, parent, manager, NULL);
+  ans = gtk_recent_chooser_dialog_new_for_manager(title, parent, manager, NULL,
+                                                  NULL);
 
   _result = PROTECT(toRPointerWithSink(ans, "GtkWidget"));
 
@@ -583,7 +597,7 @@ S_gtk_text_buffer_insert_with_tags_by_name(USER_OBJECT_ s_object, USER_OBJECT_ s
 		gtk_text_buffer_get_iter_at_offset(object, &start, start_offset);
 
 		for (i = 0; i < GET_LENGTH(s_tag_names); i++) {
-			gtk_text_buffer_apply_tag_by_name(object, (const gchar*)asCString(VECTOR_ELT(s_tag_names, i)), &start, iter);
+			gtk_text_buffer_apply_tag_by_name(object, (const gchar*)asCString(STRING_ELT(s_tag_names, i)), &start, iter);
 		}
 		
 		return(_result);
@@ -719,14 +733,11 @@ S_gtk_list_store_set_value(USER_OBJECT_ s_object, USER_OBJECT_ s_iter, USER_OBJE
 	GValue value = { 0, };
 	
 	USER_OBJECT_ _result = NULL_USER_OBJECT;
-
+        
 	g_value_init(&value, gtk_tree_model_get_column_type(GTK_TREE_MODEL(object), column));
 	R_setGValueFromSValue(&value, s_value);
-	
 	gtk_list_store_set_value(object, iter, column, &value);
-
 	CLEANUP(g_value_unset, &value);
-
 	return(_result);
 }
 
@@ -775,7 +786,7 @@ S_gtk_cell_layout_set_attributes(USER_OBJECT_ s_object, USER_OBJECT_ s_cell, USE
         USER_OBJECT_ _result = NULL_USER_OBJECT;
 
 		for (i = 0; i < GET_LENGTH(s_attributes); i++)
-			gtk_cell_layout_add_attribute(object, cell, asCString(VECTOR_ELT(names, i)), 
+			gtk_cell_layout_add_attribute(object, cell, asCString(STRING_ELT(names, i)), 
 				asCInteger(VECTOR_ELT(s_attributes, i)));
 
         return(_result);
@@ -793,7 +804,7 @@ S_gtk_container_child_set(USER_OBJECT_ s_object, USER_OBJECT_ s_child, USER_OBJE
         USER_OBJECT_ _result = NULL_USER_OBJECT;
 
 		for (i = 0; i < GET_LENGTH(s_props); i++)
-			gtk_container_child_set_property(object, child, asCString(VECTOR_ELT(names, i)), 
+			gtk_container_child_set_property(object, child, asCString(STRING_ELT(names, i)), 
 				asCGValue(VECTOR_ELT(s_props, i)));
 
         return(_result);
@@ -825,7 +836,7 @@ S_gtk_container_child_get(USER_OBJECT_ s_object, USER_OBJECT_ s_child, USER_OBJE
 		
 		for (i = 0; i < GET_LENGTH(s_names); i++)
 			SET_VECTOR_ELT(_result, i, VECTOR_ELT(
-				S_gtk_container_child_get_property(s_object, s_child, VECTOR_ELT(s_names, i)), 1));
+				S_gtk_container_child_get_property(s_object, s_child, STRING_ELT(s_names, i)), 1));
 
         return(_result);
 }
@@ -852,7 +863,7 @@ S_gtk_tree_view_insert_column_with_attributes(USER_OBJECT_ s_object, USER_OBJECT
 
 		s_names = GET_NAMES(s_attributes);
 		for (i = 0; i < GET_LENGTH(s_attributes); i++) {
-			gtk_tree_view_column_add_attribute(column, cell, CHAR_DEREF(STRING_ELT(s_names, i)), 
+			gtk_tree_view_column_add_attribute(column, cell, asCString(STRING_ELT(s_names, i)), 
 				asCInteger(VECTOR_ELT(s_attributes, i)));
 		}
 		ans = gtk_tree_view_insert_column(object, column, position);
@@ -1057,7 +1068,7 @@ S_gtk_container_child_set_property(USER_OBJECT_ s_object, USER_OBJECT_ s_child,
 
 	USER_OBJECT_ _result = NULL_USER_OBJECT;
 
-	GParamSpec *spec = gtk_container_class_find_child_property(G_OBJECT_CLASS(object), property_name);
+	GParamSpec *spec = gtk_container_class_find_child_property(G_OBJECT_GET_CLASS(object), property_name);
 	g_value_init(&value, G_PARAM_SPEC_VALUE_TYPE(spec));
 	R_setGValueFromSValue(&value, s_value);
 	
@@ -1266,6 +1277,67 @@ S_gtk_builder_connect_signals(USER_OBJECT_ s_object, USER_OBJECT_ s_user_data)
 }
 #endif
 
+
+/* Tooltip contexts due to in/out parameters */
+USER_OBJECT_
+S_gtk_tree_view_get_tooltip_context(USER_OBJECT_ s_object, USER_OBJECT_ s_x, USER_OBJECT_ s_y, USER_OBJECT_ s_keyboard_tip)
+{
+  USER_OBJECT_ _result = NULL_USER_OBJECT;
+#if GTK_CHECK_VERSION(2, 12, 0)
+  GtkTreeView* object = GTK_TREE_VIEW(getPtrValue(s_object));
+  gint x = asCInteger(s_x);
+  gint y = asCInteger(s_y);
+  gboolean keyboard_tip = ((gboolean)asCLogical(s_keyboard_tip));
+
+  gboolean ans;
+  GtkTreeModel* model = NULL;
+  GtkTreePath* path = NULL;
+  GtkTreeIter iter;
+
+  ans = gtk_tree_view_get_tooltip_context(object, &x, &y, keyboard_tip, &model, &path, &iter);
+
+  _result = asRLogical(ans);
+
+  _result = retByVal(_result, "x", asRInteger(x), "y", asRInteger(y), "model", toRPointerWithRef(model, "GtkTreeModel"), "path", toRPointerWithFinalizer(path, "GtkTreePath", (RPointerFinalizer) gtk_tree_path_free), "iter", toRPointerWithFinalizer(&iter ? gtk_tree_iter_copy(&iter) : NULL, "GtkTreeIter", (RPointerFinalizer) gtk_tree_iter_free), NULL);
+  ;
+  ;
+  ;
+#else
+  error("gtk_tree_view_get_tooltip_context exists only in Gtk >= 2.12.0");
+#endif
+
+  return(_result);
+}
+USER_OBJECT_
+S_gtk_icon_view_get_tooltip_context(USER_OBJECT_ s_object, USER_OBJECT_ s_x, USER_OBJECT_ s_y, USER_OBJECT_ s_keyboard_tip)
+{
+  USER_OBJECT_ _result = NULL_USER_OBJECT;
+#if GTK_CHECK_VERSION(2, 12, 0)
+  GtkIconView* object = GTK_ICON_VIEW(getPtrValue(s_object));
+  gint x = asCInteger(s_x);
+  gint y = asCInteger(s_y);
+  gboolean keyboard_tip = ((gboolean)asCLogical(s_keyboard_tip));
+
+  gboolean ans;
+  GtkTreeModel* model = NULL;
+  GtkTreePath* path = NULL;
+  GtkTreeIter iter;
+
+  ans = gtk_icon_view_get_tooltip_context(object, &x, &y, keyboard_tip, &model, &path, &iter);
+
+  _result = asRLogical(ans);
+
+  _result = retByVal(_result, "x", asRInteger(x), "y", asRInteger(y), "model", toRPointerWithRef(model, "GtkTreeModel"), "path", toRPointerWithFinalizer(path, "GtkTreePath", (RPointerFinalizer) gtk_tree_path_free), "iter", toRPointerWithFinalizer(&iter ? gtk_tree_iter_copy(&iter) : NULL, "GtkTreeIter", (RPointerFinalizer) gtk_tree_iter_free), NULL);
+  ;
+  ;
+  ;
+#else
+  error("gtk_icon_view_get_tooltip_context exists only in Gtk >= 2.12.0");
+#endif
+
+  return(_result);
+}
+
 /* This provides a quick way to convert GtkTreePaths to indices.
    Such an operation is necessary when interpreting selections, for example.
 */
@@ -1540,3 +1612,15 @@ S_gtk_tree_iter_get_stamp(USER_OBJECT_ s_iter)
   GtkTreeIter *iter = getPtrValue(s_iter);
   return asRInteger(iter->stamp);
 }
+
+/* get compile-time GTK version */
+
+USER_OBJECT_
+boundGTKVersion(void) {
+  USER_OBJECT_ version;
+  version = NEW_INTEGER(3);
+  INTEGER(version)[0] = GTK_MAJOR_VERSION;
+  INTEGER(version)[1] = GTK_MINOR_VERSION;
+  INTEGER(version)[2] = GTK_MICRO_VERSION;
+  return(version);
+}
